/**
 * easyWSDL - easyWSDL toolbox Platform.
 * Copyright (c) 2008,  eBM Websourcing
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.ow2.easywsdl.wsdl.api.abstractElmt;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.easywsdl.wsdl.api.WSDLReader;
import org.ow2.easywsdl.wsdl.util.WSDLJDomAnalyzer;


/**
 * @author Nicolas Salatge - eBM WebSourcing
 */
public abstract class AbstractWSDLReaderImpl implements WSDLReader {

    private static Logger log = Logger.getLogger(AbstractWSDLReaderImpl.class.getName());

    // analyze wsdl
    protected WSDLJDomAnalyzer analyzer;

    private Map<FeatureConstants, Object> features = new HashMap<FeatureConstants, Object>();

    public AbstractWSDLReaderImpl() {
        this.features.put(FeatureConstants.VERBOSE, false);
        this.features.put(FeatureConstants.IMPORT_DOCUMENTS, true);
    }

    /**
     * @return the analyzer
     */
    public final WSDLJDomAnalyzer getAnalyzer() {
        return this.analyzer;
    }

    /**
     * @param analyzer
     *            the analyzer to set
     */
    public final void setAnalyzer(final WSDLJDomAnalyzer analyzer) {
        this.analyzer = analyzer;
    }

    public final void setFeature(final FeatureConstants name, final Object value)
            throws WSDLException {
        this.features.put(name, value);
        AbstractWSDLReaderImpl.log.finest("set proterty: " + name + " - value = " + value);
    }

    public final Object getFeature(final FeatureConstants name) {
        return this.features.get(name);
    }

    public final Map<FeatureConstants, Object> getFeatures() {
        return this.features;
    }

    public final void setFeatures(final Map<FeatureConstants, Object> features) {
        this.features = features;
    }
}
