/**
 * easyWSDL - easyWSDL toolbox Platform.
 * Copyright (c) 2008,  eBM Websourcing
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.ow2.easywsdl.wsdl.decorator;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.net.URI;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.NamespaceContext;
import javax.xml.namespace.QName;

import org.ow2.easywsdl.schema.api.Documentation;
import org.ow2.easywsdl.schema.api.XmlException;
import org.ow2.easywsdl.schema.api.abstractElmt.AbstractSchemaElementImpl;
import org.ow2.easywsdl.wsdl.api.Description;
import org.ow2.easywsdl.wsdl.api.WSDLElement;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.easywsdl.wsdl.api.WSDLImportException;
import org.ow2.easywsdl.wsdl.api.abstractElmt.AbstractDescriptionImpl;
import org.ow2.easywsdl.wsdl.api.abstractElmt.AbstractWSDLElementImpl;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfBinding;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfDescription;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfEndpoint;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfImport;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfInclude;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfInterfaceType;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfService;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfTypes;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfDescription.WSDLVersionConstants;
import org.w3c.dom.Element;


/**
 * @author Nicolas Salatge - eBM WebSourcing
 */
@SuppressWarnings("unchecked")
public class DecoratorDescriptionImpl<S extends AbsItfService, E extends AbsItfEndpoint, B extends AbsItfBinding, I extends AbsItfInterfaceType, Incl extends AbsItfInclude, Impt extends AbsItfImport, T extends AbsItfTypes> extends AbstractWSDLElementImpl implements Decorator {

	/**
	 *
	 */
	private static final long serialVersionUID = 1L;

	protected AbsItfDescription<S, E, B, I, Incl, Impt, T> wsdl;

	private Class<? extends DecoratorTypesImpl> typesImpl;

	public DecoratorDescriptionImpl(final AbsItfDescription<S, E, B, I, Incl, Impt, T> wsdl, Class<? extends DecoratorTypesImpl> typesImpl)
	throws WSDLException {
		final AbsItfDescription wsdlT = wsdl;
		this.wsdl = wsdlT;
		this.typesImpl = typesImpl;
	}

	public Description getFirstDescription() {
		Description res = null;
		AbsItfDescription tmp = this.wsdl;
		while ((tmp != null) && (tmp instanceof DecoratorDescriptionImpl)) {
			tmp = ((DecoratorDescriptionImpl) tmp).getFirstDescription();
		}
		if (tmp instanceof Description) {
			res = (Description) tmp;
		}
		return res;
	}

	public void addBinding(final B arg0) {
		this.wsdl.addBinding(arg0);
	}

	public void addImport(final Impt arg0) {
		this.wsdl.addImport(arg0);
	}

	public void addInclude(final Incl arg0) throws WSDLException {
		this.wsdl.addInclude(arg0);
	}

	public void addNamespace(final String arg0, final String arg1) {
		this.wsdl.addNamespace(arg0, arg1);
	}

	public void addService(final S arg0) {
		this.wsdl.addService(arg0);
	}

	public B createBinding() {
		return this.wsdl.createBinding();
	}

	public Impt createImport() throws WSDLException, WSDLImportException {
		return this.wsdl.createImport();
	}

	public S createService() {
		return this.wsdl.createService();
	}

	public T createTypes() {
		return this.wsdl.createTypes();
	}

	public B getBinding(final QName arg0) {
		return this.wsdl.getBinding(arg0);
	}

	public List<B> getBindings() {
		return this.wsdl.getBindings();
	}

	public URI getDocumentBaseURI() {
		return this.wsdl.getDocumentBaseURI();
	}

	public List<Impt> getImports() {
		return this.wsdl.getImports();
	}

	public List<Impt> getImports(final String arg0) {
		return this.wsdl.getImports();
	}

	public List<Incl> getIncludes() {
		return this.wsdl.getIncludes();
	}

	public List<Incl> getIncludes(final String arg0) {
		return this.wsdl.getIncludes(arg0);
	}

	public NamespaceContext getNamespaces() {
		return this.wsdl.getNamespaces();
	}

	public QName getQName() throws WSDLException {
		return this.wsdl.getQName();
	}

	public Map<String, String> getSchemaLocation() {
		return this.wsdl.getSchemaLocation();
	}

	public S getService(final QName arg0) {
		return this.wsdl.getService(arg0);
	}

	public List<S> getServices() {
		return this.wsdl.getServices();
	}

	public String getTargetNamespace() {
		return this.wsdl.getTargetNamespace();
	}

	public T getTypes() {
		T types = null;
		if(typesImpl != null) {
			try {
				final Constructor c = typesImpl.getConstructors()[0];
				types = (T) c.newInstance(this.wsdl.getTypes());
			} catch (IllegalArgumentException e) {
				// do nothing
				e.printStackTrace();
			} catch (InstantiationException e) {
				// do nothing
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				// do nothing
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				// do nothing
				e.printStackTrace();
			}
		} else {
			types = this.wsdl.getTypes();
		}
		return types;
	}

	public WSDLVersionConstants getVersion() {
		return this.wsdl.getVersion();
	}

	public B removeBinding(final QName arg0) {
		return this.wsdl.removeBinding(arg0);
	}

	public Impt removeImport(final Impt arg0) {
		return this.wsdl.removeImport(arg0);
	}

	public Incl removeInclude(final Incl arg0) throws WSDLException {
		return this.wsdl.removeInclude(arg0);
	}

	public String removeNamespace(final String arg0) {
		return this.wsdl.removeNamespace(arg0);
	}

	public S removeService(final QName arg0) {
		return this.wsdl.removeService(arg0);
	}

	public void setDocumentBaseURI(final URI arg0) {
		this.wsdl.setDocumentBaseURI(arg0);
	}

	public void setQName(final QName arg0) throws WSDLException {
		this.wsdl.setQName(arg0);
	}

	public void setTargetNamespace(final String arg0) {
		this.wsdl.setTargetNamespace(arg0);
	}

	public void setTypes(final T type) {
		if((type instanceof DecoratorTypesImpl)&&(this.wsdl instanceof AbstractDescriptionImpl)) {
			((AbstractDescriptionImpl)this.wsdl).setTypes( (AbsItfTypes) ((DecoratorTypesImpl)type).getDecorator());
		} else {
			this.wsdl.setTypes(type);
		}
	}

	public Documentation createDocumentation() {
		return this.wsdl.createDocumentation();
	}

	public Documentation getDocumentation() {
		return this.wsdl.getDocumentation();
	}

	public Map<QName, String> getOtherAttributes() throws XmlException {
		return this.wsdl.getOtherAttributes();
	}

	public List<Element> getOtherElements() throws XmlException {
		return this.wsdl.getOtherElements();
	}

	public void setDocumentation(final Documentation arg0) {
		this.wsdl.setDocumentation(arg0);
	}

	public I createInterface() {
		return this.wsdl.createInterface();
	}

	public I getInterface(final QName name) {
		return this.wsdl.getInterface(name);
	}

	public List<I> getInterfaces() {
		return this.wsdl.getInterfaces();
	}

	public I removeInterface(final QName name) {
		return this.wsdl.removeInterface(name);
	}

	public void addInterface(final I interfaceType) {
		this.wsdl.addInterface(interfaceType);
	}

	public B createDefaultSoapBinding(String bindingName,
			E endpoint, I itf) {
		return this.wsdl.createDefaultSoapBinding(bindingName, endpoint, itf);
	}
	
	public List<E> findEndpointsImplementingInterface(I itf) {
		return this.wsdl.findEndpointsImplementingInterface(itf);
	}

	@Override
	public Object getModel() {
		return ((AbstractWSDLElementImpl)this.wsdl).getModel();
	}

	@Override
	public AbstractSchemaElementImpl getParent() {
		return ((AbstractWSDLElementImpl)this.wsdl).getParent();
	}

	public WSDLElement getDecorator() {
		return this.wsdl;
	}

}
