/**
 * easyWSDL - easyWSDL toolbox Platform.
 * Copyright (c) 2008,  eBM Websourcing
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.ow2.easywsdl.wsdl.impl.generic;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.transform.dom.DOMSource;

import org.jdom.JDOMException;
import org.ow2.easywsdl.schema.api.XmlException;
import org.ow2.easywsdl.schema.util.SourceHelper;
import org.ow2.easywsdl.wsdl.api.Description;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.easywsdl.wsdl.api.WSDLImportException;
import org.ow2.easywsdl.wsdl.api.abstractElmt.AbstractIncludeImpl;
import org.ow2.easywsdl.wsdl.api.abstractElmt.AbstractWSDLReaderImpl;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfDescription.WSDLVersionConstants;
import org.ow2.easywsdl.wsdl.util.WSDLJDomAnalyzer;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;

/**
 * @author Nicolas Salatge - eBM WebSourcing
 */
public class WSDLReaderImpl extends AbstractWSDLReaderImpl implements
		org.ow2.easywsdl.wsdl.api.WSDLReader {

	List<Class> addedObjectFactories = null;

	private static Logger log = Logger
			.getLogger(WSDLReaderImpl.class.getName());

	public WSDLReaderImpl() {

	}

	public WSDLReaderImpl(List<Class> addedObjectFactories) {
		this.addedObjectFactories = addedObjectFactories;
	}

	public Description readWSDL(final URI wsdlURI) throws WSDLException {
		Description desc = null;
		try {

			InputStream source = null;
			final File f = new File(wsdlURI.getPath());
			if (f.exists()) {
				source = new FileInputStream(f);
			} else {
				source = wsdlURI.toURL().openStream();
			}

			desc = this.readWSDL(wsdlURI, SourceHelper
					.convertInputSource2DOMSource(new InputSource(source)));
		} catch (final MalformedURLException e) {
			throw new WSDLException("Can not get wsdl at: " + wsdlURI, e);
		} catch (final IOException e) {
			throw new WSDLException("Can not get wsdl at: " + wsdlURI, e);
		} catch (final WSDLException e) {
			throw new WSDLException("Can not get wsdl at: " + wsdlURI, e);
		} catch (final XmlException e) {
			throw new WSDLException("Can not get wsdl at: " + wsdlURI, e);
		}
		return desc;

	}

	private org.ow2.easywsdl.wsdl.api.WSDLReader getConcreteReader(
			final DOMSource input) throws WSDLException {
		org.ow2.easywsdl.wsdl.api.WSDLReader reader = null;
		try {
			// analyze wsdl
			this.analyzer = new WSDLJDomAnalyzer(SourceHelper
					.convertDOMSource2InputSource(input));

			final WSDLVersionConstants version = this.analyzer.getVersion();

			if (version == WSDLVersionConstants.WSDL11) {
				reader = new org.ow2.easywsdl.wsdl.impl.wsdl11.WSDLReaderImpl(
						this.addedObjectFactories);
			} else if (version == WSDLVersionConstants.WSDL20) {
				reader = new org.ow2.easywsdl.wsdl.impl.wsdl20.WSDLReaderImpl(
						this.addedObjectFactories);
			} else {
				throw new WSDLException("unknown version of wsdl");
			}
			((AbstractWSDLReaderImpl) reader).setFeatures(this.getFeatures());
		} catch (final JDOMException e) {
			throw new WSDLException("unknown version of wsdl", e);
		} catch (final XmlException e) {
			throw new WSDLException("unknown version of wsdl", e);
		}
		return reader;
	}

	public Description readWSDL(final Document wsdlDocument,
			Map<URI, Document> imports) throws WSDLException {
		Description desc = null;
		final DOMSource domSource = new DOMSource(wsdlDocument);
		try {
			// Convert DOM imports to DOMSource import
			Map<URI, DOMSource> importSources = new HashMap<URI, DOMSource>();
			if (imports != null && imports.size() > 0) {
				Set<URI> keys = imports.keySet();
				for (URI key : keys) {
					importSources.put(key, new DOMSource(imports.get(key)));
				}
			}
			URI uri = null;
			if (wsdlDocument.getDocumentURI() != null) {
				uri = new URI(wsdlDocument.getDocumentURI());
			}
			desc = this.readWSDL(uri, domSource, importSources);
		} catch (final URISyntaxException e) {
			throw new WSDLException(e);
		}
		return desc;
	}

	public Description readWSDL(final URI wsdlURI, final DOMSource inputSource,
			Map<URI, DOMSource> imports) throws WSDLException {
		Description desc = null;

		final DOMSource source1;
		final DOMSource source2;
		try {
			source1 = SourceHelper.cloneSource(inputSource);
			source2 = SourceHelper.cloneSource(inputSource);
		} catch (XmlException e) {
			throw new WSDLException(e);
		}
		// analyze wsdl
		final org.ow2.easywsdl.wsdl.api.WSDLReader reader = this
				.getConcreteReader(source1);
		((AbstractWSDLReaderImpl) reader).setAnalyzer(this.analyzer);

		desc = reader.readWSDL(wsdlURI, source2, imports);

		return desc;
	}

	public Description readWSDL(Document wsdlDocument) throws WSDLException {
		return this.readWSDL(wsdlDocument, null);
	}

	public Description readWSDL(final URI wsdlURI, final DOMSource inputSource)
			throws WSDLException {
		return this.readWSDL(wsdlURI, inputSource, null);
	}

	public Description readWSDL(URI wsdlURI, DOMSource inputSource,
			Map<URI, DOMSource> imports, int importDepth) throws WSDLException {
		// MUST NOT BE CALLED
		// TODO refactor easy wsdl to make this method protected
		throw new WSDLException("This method must not be called externally");
	}

}
