/**
 * easyWSDL - easyWSDL toolbox Platform.
 * Copyright (c) 2008,  eBM Websourcing
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.ow2.easywsdl.wsdl.impl.wsdl20;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.xml.transform.dom.DOMSource;

import org.apache.commons.lang.NotImplementedException;
import org.jdom.Document;
import org.ow2.easywsdl.schema.SchemaFactory;
import org.ow2.easywsdl.schema.api.Import;
import org.ow2.easywsdl.schema.api.Schema;
import org.ow2.easywsdl.schema.api.SchemaException;
import org.ow2.easywsdl.schema.api.XmlException;
import org.ow2.easywsdl.schema.api.SchemaReader.FeatureConstants;
import org.ow2.easywsdl.schema.api.abstractElmt.AbstractSchemaElementImpl;
import org.ow2.easywsdl.schema.api.abstractElmt.AbstractSchemaImpl;
import org.ow2.easywsdl.wsdl.api.Types;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.easywsdl.wsdl.api.abstractElmt.AbstractTypesImpl;
import org.ow2.easywsdl.wsdl.org.w3.ns.wsdl.TypesType;
import org.ow2.easywsdl.wsdl.org.w3.ns.wsdl.TypesType.AnyItem;
import org.ow2.easywsdl.wsdl.util.Util;
import org.w3c.dom.Element;

/**
 * @author Nicolas Salatge - eBM WebSourcing
 */
public class TypesImpl extends AbstractTypesImpl<TypesType, Schema, Import>
		implements Types {

	/**
	 *
	 */
	private static final long serialVersionUID = 1L;

	public TypesImpl(final TypesType types, final DescriptionImpl desc,
			final Map<URI, DOMSource> imports) throws WSDLException {
		super(types, desc);
		this.desc = desc;

		// get the documentation
		this.documentation = new org.ow2.easywsdl.wsdl.impl.wsdl20.DocumentationImpl(
				this.model.getDocumentation(), this);

		final List<org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Schema> scs = new ArrayList<org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Schema>();
		for (final AnyItem item : this.model.getAnyItems()) {

			if (item.getItem() instanceof org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Schema) {
				scs
						.add((org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Schema) item
								.getItem());
			}

			if (item.getItem() instanceof org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Import) {
				org.ow2.easywsdl.schema.api.Import impt = null;
				try {
					impt = new org.ow2.easywsdl.schema.impl.ImportImpl(
							this.desc.getDocumentBaseURI(),
							(org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Import) item
									.getItem(),
							Util.convertWSDLFeatures2SchemaFeature(this.desc),
							imports);
				} catch (SchemaException e) {
					throw new WSDLException(e);
				}
				this.importedSchemas.add(impt);
			}
		}

		this.addImportedSchemasInAllList();

		final List<Document> schemaDocuments = this.getSchemaDocuments();

		if (schemaDocuments.size() != (scs.size() + this.importedSchemas.size())) {
			throw new WSDLException("Error in validation of types");
		}

		final Map<FeatureConstants, Object> features = Util
				.convertWSDLFeatures2SchemaFeature(this.desc);

		try {
			int i = 0;
			for (final org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Schema schema : scs) {
				this.schemas.add(new org.ow2.easywsdl.schema.impl.SchemaImpl(
						this.desc.getDocumentBaseURI(), schema, schemaDocuments
								.get(i), features, imports));
				i++;
			}
		} catch (final SchemaException e) {
			throw new WSDLException(e);
		}

		this.setAllNamespacesInAllSchemas();
	}

	@Override
	public List<Element> getOtherElements() throws XmlException {
		final List<Element> res = super.getOtherElements();

		for (final Object item : this.model.getAny()) {
			if (item instanceof Element) {
				res.add((Element) item);
			}
		}

		return res;
	}

	public Schema createSchema() {
		Schema schema = null;
		try {
			schema = SchemaFactory.newInstance().newSchema();
		} catch (SchemaException e) {
			// Do nothing
			e.printStackTrace();
		}
		((AbstractSchemaImpl) schema).setParent(this);
		return schema;
	}

	@Override
	public void addSchema(Schema schema) {
		AnyItem item = new AnyItem();
		item.setItem(((AbstractSchemaElementImpl) schema).getModel());
		this.model.getAnyItems().add(item);
		super.addSchema(schema);
	}

	public Schema removeSchema() {
		throw new NotImplementedException();
	}
}
