/**
 * easyWSDL - easyWSDL toolbox Platform.
 * Copyright (c) 2008,  eBM Websourcing
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.ow2.easywsdl.wsdl.impl.wsdl20;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;

import org.jdom.JDOMException;
import org.ow2.easywsdl.schema.api.XmlException;
import org.ow2.easywsdl.schema.util.SourceHelper;
import org.ow2.easywsdl.wsdl.api.Description;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.easywsdl.wsdl.api.WSDLImportException;
import org.ow2.easywsdl.wsdl.api.abstractElmt.AbstractWSDLReaderImpl;
import org.ow2.easywsdl.wsdl.org.w3.ns.wsdl.DescriptionType;
import org.ow2.easywsdl.wsdl.util.WSDLJDomAnalyzer;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;

/**
 * @author Nicolas Salatge - eBM WebSourcing
 */
public class WSDLReaderImpl extends AbstractWSDLReaderImpl implements
		org.ow2.easywsdl.wsdl.api.WSDLReader {

	/**
	 * The JAXB unique unmarshaller.
	 */
	private Unmarshaller unmarshaller = null;

	private static Logger log = Logger.getLogger(WSDLReaderImpl.class.getName());
	
	/*
	 * Private object initializations
	 */
	public WSDLReaderImpl() throws WSDLException {
		try {
			WSDLJAXBContext.createDefaultContext();

			unmarshaller = WSDLJAXBContext.getJaxbContext()
					.createUnmarshaller();
			// unmarshaller.setSchema(schema);

		} catch (final JAXBException e) {
			throw new WSDLException(e);
		} catch (final WSDLException e) {
			throw new WSDLException(e);
		}
	}

	public WSDLReaderImpl(List<Class> addedObjectFactories)
			throws WSDLException {
		try {
			WSDLJAXBContext
					.createContextWithOtherObjectFactories(addedObjectFactories);

			unmarshaller = WSDLJAXBContext.getJaxbContext()
					.createUnmarshaller();
			// unmarshaller.setSchema(schema);

		} catch (final JAXBException e) {
			throw new WSDLException(e);
		} catch (final WSDLException e) {
			throw new WSDLException(e);
		}
	}

	/**
	 * Build the Java bindings from the WSDL descriptor in XML form. The
	 * validation against the WSDL schema is processed during the
	 * transformation.
	 * 
	 * @param AgreementDescriptorStream
	 *            The Agreement descriptor in XML form
	 * @return The root object of the EndpointReference descriptor
	 * @throws WSDLException
	 *             The exception raised during the unmarshaller creation or the
	 *             exception raised during the build of the java bindings.
	 */
	private DescriptionType convertStream2WSDL20Description(
			final Source wsdlDescriptorStream) throws WSDLException {

		try {
			// TODO : Check if it is a Thread safe method
			final JAXBElement<DescriptionType> wsdlBinding = this
					.getUnMarshaller().unmarshal(wsdlDescriptorStream,
							DescriptionType.class);

			return wsdlBinding.getValue();

		} catch (final JAXBException e) {
			throw new WSDLException(
					"Failed to build Java bindings from WSDL descriptor XML document",
					e);
		}
	}

	/**
	 * Get the unmarshaller instance.
	 * 
	 * @return the unmarshaller instance
	 * @throws WSDLException
	 *             The exception raised during the unmarshaller creation.
	 */
	public Unmarshaller getUnMarshaller() throws WSDLException {
		return this.unmarshaller;
	}

	public Description readWSDL(final URI wsdlURI) throws WSDLException {
		Description desc = null;
		try {
			InputStream input = null;

			File f = null;
            if(wsdlURI.getPath() != null) {
            	f = new File(wsdlURI.getPath());
            }
            if (f != null) {
				input = new FileInputStream(f);
			} else {
				input = wsdlURI.toURL().openStream();
			}
			desc = this.readWSDL(wsdlURI, SourceHelper
					.convertInputSource2DOMSource(new InputSource(input)));
		} catch (final MalformedURLException e) {
			throw new WSDLException("Can not get wsdl at: " + wsdlURI, e);
		} catch (final IOException e) {
			throw new WSDLException("Can not get wsdl at: " + wsdlURI, e);
		} catch (final WSDLException e) {
			throw new WSDLException("Can not get wsdl at: " + wsdlURI, e);
		} catch (final XmlException e) {
			throw new WSDLException("Can not get wsdl at: " + wsdlURI, e);
		}
		return desc;
	}

	public Description readWSDL(final Document wsdlDocument,
			Map<URI, Document> imports) throws WSDLException {
		Description desc = null;
		try {
			// Convert DOM imports to DOMSource import
			Map<URI, DOMSource> importSources = new HashMap<URI, DOMSource>();
			if (imports != null && imports.size() > 0) {
				Set<URI> keys = imports.keySet();
				for (URI key : keys) {
					importSources.put(key, new DOMSource(imports.get(key)));
				}
			}
			// Read WSDL
			desc = this.readWSDL(new URI(wsdlDocument.getDocumentURI()),
					new DOMSource(wsdlDocument));
		} catch (final URISyntaxException e) {
			throw new WSDLException(e);
		}
		return desc;
	}
	
	public Description readWSDL(final URI wsdlURI, final DOMSource inputSource,
			Map<URI, DOMSource> imports) throws WSDLException {
		return readWSDL(wsdlURI, inputSource, imports, 0);
	}

	public Description readWSDL(final URI wsdlURI, final DOMSource inputSource,
			Map<URI, DOMSource> imports, int importDepth) throws WSDLException {
		Description desc = null;
		try {

			if (this.analyzer == null) {
				final DOMSource sourcebis = SourceHelper
						.cloneSource(inputSource);
				this.analyzer = new WSDLJDomAnalyzer(SourceHelper
						.convertDOMSource2InputSource(sourcebis));
			}
			final DOMSource source = SourceHelper.cloneSource(inputSource);
			final DescriptionType def = this
					.convertStream2WSDL20Description(source);
			desc = new org.ow2.easywsdl.wsdl.impl.wsdl20.DescriptionImpl(
					wsdlURI, def, this.analyzer.getDocument(), this
							.getFeatures(), imports, importDepth);
		} catch (final JDOMException e) {
			throw new WSDLException(e);
		} catch (final XmlException e) {
			throw new WSDLException(e);
		} catch (WSDLImportException e) {
			throw new WSDLException("Can't parse all imports", e);
		}
		return desc;
	}

	public Description readWSDL(Document wsdlDocument) throws WSDLException {
		return this.readWSDL(wsdlDocument, null);
	}

	public Description readWSDL(final URI wsdlURI, final DOMSource inputSource)
			throws WSDLException {
		return this.readWSDL(wsdlURI, inputSource, null, 0);
	}

}
